"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticWebsite = void 0;
const aws_cloudfront_1 = require("aws-cdk-lib/aws-cloudfront");
const constructs_1 = require("constructs");
const aws_cdk_lib_1 = require("aws-cdk-lib");
const aws_s3_1 = require("aws-cdk-lib/aws-s3");
const aws_route53_targets_1 = require("aws-cdk-lib/aws-route53-targets");
const aws_s3_deployment_1 = require("aws-cdk-lib/aws-s3-deployment");
const aws_route53_1 = require("aws-cdk-lib/aws-route53");
/**
 * Static site infrastructure, which deploys site content to an S3 bucket.
 *
 * The site redirects from HTTP to HTTPS, using a CloudFront distribution,
 * Route53 alias record, and ACM certificate.
 */
class StaticWebsite extends constructs_1.Construct {
    constructor(parent, name, props) {
        super(parent, name);
        let zone;
        if (props.hostedZoneId) {
            zone = aws_route53_1.HostedZone.fromHostedZoneAttributes(this, 'Zone', {
                zoneName: props.domainName,
                hostedZoneId: props.hostedZoneId,
            });
        }
        else {
            zone = aws_route53_1.HostedZone.fromLookup(this, 'Zone', {
                domainName: props.domainName,
            });
        }
        new aws_cdk_lib_1.CfnOutput(this, 'Site', {
            value: 'https://' + props.siteSubDomain + '.' + props.domainName,
        });
        // Content bucket
        const siteBucket = new aws_s3_1.Bucket(this, 'SiteBucket', {
            bucketName: props.siteSubDomain + '.' + props.domainName,
            websiteIndexDocument: 'index.html',
            websiteErrorDocument: 'index.html',
            publicReadAccess: true,
            removalPolicy: aws_cdk_lib_1.RemovalPolicy.DESTROY,
        });
        const redirectSiteBucket = new aws_s3_1.Bucket(this, 'RedirectSiteBucket', {
            bucketName: props.domainName,
            websiteRedirect: {
                hostName: props.siteSubDomain + '.' + props.domainName,
                protocol: aws_s3_1.RedirectProtocol.HTTPS,
            },
            removalPolicy: aws_cdk_lib_1.RemovalPolicy.DESTROY,
        });
        // TLS certificate
        // const certificateArn = new DnsValidatedCertificate(this, 'SiteCertificate', {
        //   domainName: props.siteSubDomain + '.' + props.domainName,
        //   subjectAlternativeNames: [props.domainName],
        //   hostedZone: zone,
        //   region: 'us-east-1', // Cloudfront only checks this region for certificates.
        // }).certificateArn
        // new CfnOutput(this, 'Certificate', { value: certificateArn })
        const viewerCertificateSubdomain = aws_cloudfront_1.ViewerCertificate.fromIamCertificate('MYIAMROLEIDENTIFIER', {
            aliases: [props.siteSubDomain + '.' + props.domainName],
            sslMethod: aws_cloudfront_1.SSLMethod.SNI,
            securityPolicy: aws_cloudfront_1.SecurityPolicyProtocol.TLS_V1_1_2016,
        });
        // CloudFront distribution that provides HTTPS
        const distribution = new aws_cloudfront_1.CloudFrontWebDistribution(this, 'SiteDistribution', {
            viewerCertificate: viewerCertificateSubdomain,
            originConfigs: [
                {
                    customOriginSource: {
                        domainName: siteBucket.bucketWebsiteDomainName,
                        originProtocolPolicy: aws_cloudfront_1.OriginProtocolPolicy.HTTP_ONLY,
                    },
                    behaviors: [{ isDefaultBehavior: true }],
                },
            ],
        });
        new aws_cdk_lib_1.CfnOutput(this, 'DistributionId', {
            value: distribution.distributionId,
        });
        const viewerCertificate = aws_cloudfront_1.ViewerCertificate.fromIamCertificate('MYIAMROLEIDENTIFIER', {
            aliases: [props.domainName],
            sslMethod: aws_cloudfront_1.SSLMethod.SNI,
            securityPolicy: aws_cloudfront_1.SecurityPolicyProtocol.TLS_V1_1_2016,
        });
        const redirectDistribution = new aws_cloudfront_1.CloudFrontWebDistribution(this, 'RedirectSiteDistribution', {
            viewerCertificate,
            viewerProtocolPolicy: aws_cloudfront_1.ViewerProtocolPolicy.REDIRECT_TO_HTTPS,
            originConfigs: [
                {
                    customOriginSource: {
                        domainName: redirectSiteBucket.bucketWebsiteDomainName,
                        originProtocolPolicy: aws_cloudfront_1.OriginProtocolPolicy.HTTP_ONLY,
                    },
                    behaviors: [
                        {
                            isDefaultBehavior: true,
                            allowedMethods: aws_cloudfront_1.CloudFrontAllowedMethods.GET_HEAD_OPTIONS,
                        },
                    ],
                },
            ],
        });
        new aws_cdk_lib_1.CfnOutput(this, 'RedirectDistributionId', {
            value: redirectDistribution.distributionId,
        });
        // Route53 alias record for the CloudFront distribution
        new aws_route53_1.ARecord(this, 'SiteAliasRecord', {
            recordName: props.siteSubDomain + '.' + props.domainName,
            target: aws_route53_1.RecordTarget.fromAlias(new aws_route53_targets_1.CloudFrontTarget(distribution)),
            zone,
        });
        new aws_route53_1.ARecord(this, 'RedirectSiteAliasRecord', {
            recordName: props.domainName,
            target: aws_route53_1.RecordTarget.fromAlias(new aws_route53_targets_1.CloudFrontTarget(redirectDistribution)),
            zone,
        });
        // Deploy site contents to S3 bucket
        new aws_s3_deployment_1.BucketDeployment(this, 'DeployWithInvalidation', {
            sources: [aws_s3_deployment_1.Source.asset(props.assets)],
            destinationBucket: siteBucket,
            distribution,
            distributionPaths: ['/*'],
        });
    }
}
exports.StaticWebsite = StaticWebsite;
//# sourceMappingURL=cdk-construct.js.map